#pragma TextEncoding = "UTF-8"
#pragma rtGlobals=3				// Use modern global access method and strict wave access
#pragma DefaultTab={3,20,4}		// Set default tab width in Igor Pro 9 and later

//___________________________________________________________________________________________________	
//	string2text2char2byte
//	A library of functions for manipulating string, character, text, and byte data
//___________________________________________________________________________________________________	
//
//	Copyright ©2021
//	Chris Reed (FAAM Airborne Laboratory, Building 146, Central Avenue, Cranfield University, Bedfordshire, MK43 0AL, UK)
//	chris.reed@faam.ac.uk
//___________________________________________________________________________________________________	
//
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 2 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program; if not, write to the Free Software
//	Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
//___________________________________________________________________________________________________

Static Constant padValue = 0x00						// padding character for CHAR datasets

//___________________________________________________________________________________________________	
// convert a byte wave into a text wave
Function/WAVE byte2text(Wave byteWv)

	Wave/T charWv = byte2char(byteWv)
	Wave/T textWv = char2text(charWv)

	return textWv
End
//___________________________________________________________________________________________________	
// convert a char wave into a text wave
Function/WAVE char2text(Wave/T charWv)
	
	double i,j=DimSize(charWv,0)

	Make /O /FREE /T /N=(j) textWv
	
	for (i=0;i<j;i++)
		Duplicate /O /FREE /RMD=[i][] charWV strWv
		textWv[i] = UnPadString(char2Str(strWv),padValue)
	endfor

	return textWv 
End
//___________________________________________________________________________________________________	
// convert a wave of byte (2d) to text wave
Function/WAVE byte2char(Wave byteWv)

	Make /O /FREE /T=1 /N=(DimSize(byteWv,0),DimSize(byteWv,1)) charWv = num2char(byteWv[p][q])

	return charWv
End
//___________________________________________________________________________________________________	
// convert a wave of char to a string
Function/S byte2string(Wave byteWv)

	String rtnStr
	Wave charWv = byte2char(byteWv)
	
	wfprintf rtnStr,"%s" charWv
	
	return rtnStr
End
//___________________________________________________________________________________________________	
// convert a string into an array of char
Function/WAVE string2char(String str)

	Make /O /FREE /T=1 /N=(strlen(str)) charWv = str[p]
	
	return charWv
End
//___________________________________________________________________________________________________	
// convert a char to a string
Function/S char2Str(Wave/T charWv)

	String rtnStr
	wfprintf rtnStr,"%s" charWv
	
	return rtnStr
End
//___________________________________________________________________________________________________	
// get maximum string length in a text wave
Function getMaxStrLen(Wave/T Wv)

	Make /D /FREE /N=(numpnts(Wv)) lenWv = strlen(Wv[p])
	
	return WaveMax(lenWv)
End
//___________________________________________________________________________________________________
// get avgerage string length in a text wave
Function [int maxlen,double avgLen] getMaxAvgStrLen(Wave/T Wv)

	Make /D /FREE /N=(numpnts(Wv)) lenWv = strlen(Wv[p])
	WaveStats /Q /M=1 lenWv
	
	return [V_max,V_avg]
End
//___________________________________________________________________________________________________	
// convert a text wave into an array of char
Function/WAVE text2char(Wave/T textWv)
	
	int n = numpnts(textWv),maxLen = getMaxStrLen(textWv)
	
	Make /O /FREE /T=1 /N=(n,maxLen) charWv = PadString(textWv[p],maxLen,padValue)[q]

	return charWV
End
//___________________________________________________________________________________________________	
// convert a text wave into a byte wave
Function/WAVE text2byte(Wave/T textWV)
	
	Wave/T charWv = text2char(textWv)
	Wave byteWv = char2byte(charWv)

	return byteWv
End
//___________________________________________________________________________________________________	
// convert a text wave into a byte wave
Function/WAVE char2byte(Wave/T charWv)
	
	Make /O /FREE /B /N=(DimSize(charWv,0),DimSize(charWv,1)) byteWv = char2num(charWv[p][q])

	return byteWv
End
//___________________________________________________________________________________________________